from httpx import HTTPError


class OpenAPIException(Exception):
    pass


class DisallowedHost(OpenAPIException):
    pass


class NotFoundPath(OpenAPIException):
    pass


class OpenAPIHttpError(OpenAPIException, HTTPError):
    pass



from xml.etree import ElementTree as EleTree


def dict_to_xml(data):
    xml_list = ['<xml>']
    for k, v in data.items():
        if not v:
            continue

        if str(v).isdigit():
            xml_list.append('<{0}>{1}</{0}>'.format(k, v))
        else:
            xml_list.append('<{0}><![CDATA[{1}]]></{0}>'.format(k, v))
    xml_list.append('</xml>')
    return ''.join(xml_list)


def xml_to_dict(xml_string):
    try:
        return dict((child.tag, child.text) for child in EleTree.fromstring(xml_string))
    except EleTree.ParseError:
        return {}



import time
import uuid
import hmac
import base64
import inflection
from typing import Optional, Union
from urllib.parse import quote

from openapi.enums import TextChoices
from openapi.providers.base import BaseClient, BaseResult


class Code(TextChoices):
    SUCCESS = 'OK', '成功'
    FAIL = 'FAIL', '失败'


class Result(BaseResult):
    request_id: str
    biz_id: Optional[str]
    code: Union[Code, str] = Code.SUCCESS


class Client(BaseClient):
    NAME = '阿里云'
    VERSION = ''
    API_BASE_URL = 'http://{}.aliyuncs.com'

    def __init__(self, app_id, secret):
        super().__init__()
        self.app_id = app_id
        self.secret = secret
        self.codes = Code

    def request(
        self, method, prefix, action, version,
        params=None, data=None
    ):
        public_params = {
            'Action': action,
            'Version': version,
            'Timestamp': time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()),
            'SignatureMethod': 'HMAC-SHA1',
            'SignatureVersion': '1.0',
            'SignatureNonce': uuid.uuid4().hex,
            'AccessKeyId': self.app_id,
            'Format': 'JSON',
        }
        if method == 'get':
            params.update(**public_params)
        else:
            data.update(**public_params)

        sorted_data = sorted((params if method == 'get' else data).items(), key=lambda item: item[0])
        sign_string = ''
        for k, v in sorted_data:
            if not v:
                continue
            sign_string += quote(k, safe='~') + "=" + quote(v, safe='~') + '&'

        secret = '{}&'.format(self.secret)
        hmb = hmac.new(
            secret.encode(),
            ('GET&%2F&' + quote(sign_string[:-1], safe='~')).encode(), 'sha1'
        ).digest()
        signature = quote(base64.standard_b64encode(hmb).decode('ascii'), safe='~')
        query_string = f'{sign_string[:-1]}&Signature={signature}'
        request_url = f'{self.API_BASE_URL.format(prefix)}?{query_string}'
        response = self._request(method, request_url)
        if not response:
            return Result(code=self.codes.FAIL)

        result = {inflection.underscore(k): v for k, v in response.json().items()}
        return Result(**result, data=result)

    def fetch_access_token(self):
        pass



import openpyxl
from collections import namedtuple

from openapi.providers.crm.tanmarket import Client

from examples.config import config

Customer = namedtuple(
    'Customer',
    'type date reason belong lead created_at contact_at recently_at paid_at python_camp mobile '
    'username wechat memo1 memo2'
)

tan_market_config = config['tanmarket']
filepath = tan_market_config.pop('filepath')
client = Client(**tan_market_config)
client.add_webhook(config['openapi_webhook'])

workbook = openpyxl.load_workbook(filepath)
sheet = workbook.active
counter = 0
customers = []

for row in sheet.iter_rows(min_row=11296, values_only=True):
    customer = Customer(*row)
    if '删除' in customer.type:
        continue

    counter += 1
    result = client.request('post', '/v3/save-highseas', json=[
        {
            'fields': [
                # 姓名
                {
                    'id': 104220,
                    'value': [customer.username]
                },
                {
                    'id': 105086,
                    'value': [customer.created_at]
                },
                {
                    'id': 104217,
                    'value': [customer.mobile]
                },
                {
                    'id': 105739,
                    'value': [customer.python_camp or '']
                },
                {
                    'id': 104219,
                    'value': [
                        f'最近归属人：{customer.belong}, 微信：{customer.wechat or ""}'
                        f'{customer.memo1 or ""}, {customer.memo2 or ""}'
                    ]
                }
            ]
        }
    ])
    print(result)
    customers = []
    print('>>>', counter, customer)
print(counter)



# 496831983505792

from openapi.providers.crm.tanmarket import Client

from examples.config import config

c = config['tanmarket']
c.pop('filepath')
client = Client(**c)
client.add_webhook(config['openapi_webhook'])

# print(client.request('post', '/v3/common/customer/info', json={
#     'field': {
#         'fieldId': '微信号', 'values': '123'
#     }
# }))

# result = client.request('post', '/v3/clue/edit-field', json={
#     'customerId': 496831983505792,
#     'field': '描述', 'valueAsText': '哭哭哭'
# })
# print(result)

# result = client.request('post', '/v3/customer-relation/edit-field', json={
#     'customerId': 496831983505792, 'qwUserId': 'woJEYhCQAAG_I2LlJHPlfp9xJyZDfsGw',
#     'field': '描述', 'valueAsText': '嗷嗷嗷'
# })
# print(result)
#
# result = client.request('post', '/v3/customer/list-customer-of-search', json={
#     'customerId': 496831983505792
# })
# print(result)

# result = client.request('post', '/v3/common/user/info', json={
#     'mobile': 18803561683
# })
# print(result)


result = client.request('post', '/v3/customer/list-customer-of-search', json={
    'searchKey': 18099050926
})
print(result)
for item in result.data['data']:
    # print(item.keys())
    print(item.keys())
    print(item)

# Using
# result = client.request('post', '/v3/common/customer/info', json={
#     'field': {
#         'fieldId': '106074',
#         'values': '87a6f15cade25330062d363ef67a3382'
#     }
# })
# print(result)



import secrets
import hashlib
import time
import typing
from typing import Dict, Optional
from pathlib import Path

from openapi.enums import TextChoices
from openapi.providers.base import BaseClient, BaseResult
from openapi.utils import xml_to_dict, dict_to_xml


class Code(TextChoices):
    SUCCESS = 'SUCCESS', '成功'
    FAIL = 'FAIL', '失败'


class Result(BaseResult):
    return_code: str
    return_msg: str
    result_code: Optional[str]
    data: Optional[Dict]


def calculate_signature(params, api_key, sign_type='MD5'):
    if sign_type == 'MD5':
        h = hashlib.md5()
    else:
        h = hashlib.md5()

    data = [f'{k}={params[k]}' for k in sorted(params) if params[k]]
    if api_key:
        data.append('key={0}'.format(api_key))
    h.update(bytes('&'.join(data), encoding='utf-8'))
    return h.hexdigest().upper()


class Client(BaseClient):
    NAME = '微信支付'
    API_BASE_URL = 'https://api.mch.weixin.qq.com'
    API_VERSION = ''

    def __init__(
        self, app_id, mch_id, api_key_path, is_sandbox
    ):
        super().__init__()
        self.app_id = app_id
        self.mch_id = mch_id
        self.api_key_path = api_key_path
        self.is_sandbox = is_sandbox

        self.codes = Code

        with open(Path(api_key_path)) as fd:
            self.api_key = fd.read().replace('\r\n', '')

    @property
    def debug_api_key(self):
        return ''

    def request(
        self, method: str, endpoint: str,
        params: Dict = None, data: Dict = None
    ):
        request_url = f'{self.API_BASE_URL}{"/sandboxnew" if self.is_sandbox else ""}{endpoint}'

        if data:
            data['appid'] = self.app_id
            data['mch_id'] = self.mch_id

            if 'nonce_str' not in data:
                data['nonce_str'] = secrets.token_hex(16)

            data['sign'] = calculate_signature(
                data, self.debug_api_key if self.is_sandbox else self.api_key
            )

        response = self._request(
            method, request_url,
            params=params, data=dict_to_xml(data).encode('utf-8')
        )
        if response is None:
            return Result(return_code=self.codes.FAIL)
        result = xml_to_dict(response.content)
        return Result(
            return_msg=result['return_msg'],
            return_code=result['return_code'],
            result_code=result.get('result_code'),
            data=result
        )

    def get_jsapi_data(self, prepay_id):
        data = {
            'appId': self.app_id,
            'timeStamp': str(int(time.time())),
            'nonceStr': secrets.token_hex(16),
            'signType': 'MD5',
            'package': f'prepay_id={prepay_id}'
        }
        sign = calculate_signature(
            data,
            self.api_key if not self.is_sandbox else self.debug_api_key
        )
        data['paySign'] = sign.upper()
        return data

    def check_signature(self, data: typing.Dict) -> bool:
        return data['sign'] == calculate_signature(
            {
                k: v for k, v in data.items()
                if k not in ('sign',)
            },
            self.api_key if not self.is_sandbox else self.debug_api_key
        )

    def fetch_access_token(self):
        pass



from openapi.providers.feishu.bot import Client

from examples.config import config

bot = Client()
result = bot.request(
    'post',
    config['feishu']['bot'],
    json={
        'msg_type': 'text',
        'content': {'text': 'request example'}
    }
)
print(result)



import json as _json
import httpx
from datetime import datetime
from typing import Optional, Union, List, Dict

from pydantic import BaseModel, Field


MESSAGE_TEMPLATE = """date: {date}
method: {method}
request-url: {endpoint}
headers: 
{headers}
params: 
{params}
data:
{data}
result:
{result}
is_error: {is_error}
errmsg: {errmsg}
"""

SENSITIVE_KEYS = ['app_key', 'secret', 'secret_key', 'signature']


def mask_sensitive_data(data):
    if type(data) != dict:
        return data

    for key, value in data.items():
        if key in SENSITIVE_KEYS:
            data[key] = "***FILTERED***"

        if type(value) == dict:
            data[key] = mask_sensitive_data(data[key])

        if type(value) == list:
            data[key] = [mask_sensitive_data(item) for item in data[key]]

    return data


class Token(BaseModel):
    access_token: str
    refresh_token: Optional[str]
    expires_in: int = 7200
    created_at: datetime = Field(default_factory=datetime.now)

    @property
    def is_valid(self):
        return (datetime.now() - self.created_at).total_seconds() < (self.expires_in - self.expires_in * 0.3)


class BaseResult(BaseModel):
    code: Optional[int]
    data: Optional[Union[List, Dict, int]]
    message: Optional[str]


class BaseClient:
    NAME = 'Unknown Client'
    API_BASE_URL = ''
    API_VERSION = ''

    def __init__(self):
        self._ignore_endpoints = []
        self._token: Optional[Token] = None
        self.enable_webhook = False
        self.webhook_url = None

    def _request(
        self, method, request_url,
        params=None, data=None, json=None, headers=None
    ) -> httpx.Response:
        response = None
        is_error = False
        errmsg = ''
        try:
            response = httpx.request(
                method, request_url, headers=headers,
                params=params, data=data, json=json
            )
            is_error = response.is_error
        except httpx.HTTPError as exc:
            is_error = True
            errmsg = str(exc)
        finally:
            _data = data or json or {}
            format_data = {
                'date': datetime.now().strftime('%Y-%m-%d %X'),
                'method': method, 'endpoint': request_url,
                'headers': _json.dumps(
                    mask_sensitive_data(headers or {}), indent=2, ensure_ascii=False
                ),
                'params': _json.dumps(
                    mask_sensitive_data(params or {}), indent=2, ensure_ascii=False
                ),
                'data': _json.dumps(
                    mask_sensitive_data(_data),
                    indent=2, ensure_ascii=False
                ) if isinstance(_data, dict) else _data,
                'result': response.content,
                'is_error': is_error, 'errmsg': errmsg
            }
            endpoint = request_url.replace(self.API_BASE_URL, '')
            if self.enable_webhook and self.webhook_url and endpoint not in self._ignore_endpoints:
                self.send_webhook_message(MESSAGE_TEMPLATE.format(**format_data))

        return response

    def fetch_access_token(self):
        raise NotImplementedError()

    def request(self, *args, **kwargs):
        raise NotImplementedError()

    def send_webhook_message(self, message):
        try:
            httpx.request('post', self.webhook_url, json={
                'msg_type': 'post', 'content': {
                    'post': {
                        'zh_cn': {
                            'title': self.NAME,
                            'content': [
                                [{'tag': 'text', 'text': message}],
                            ]
                        }
                    }
                }
            })
        except httpx.HTTPError:
            pass

    def check_token(self, access_token):
        return True

    def refresh_access_token(self):
        pass

    @property
    def access_token(self):
        if not (
            self._token and self._token.is_valid
            and self.check_token(self._token.access_token)
        ):
            self.fetch_access_token()

        return self._token.access_token

    def add_webhook(self, webhook_url, ignore_endpoints: Optional[List] = None):
        self.enable_webhook = True
        self.webhook_url = webhook_url
        if ignore_endpoints:
            self._ignore_endpoints = ignore_endpoints
